import { clusterContexts, bucketRegions, bucketEnvironments } from "./constants.js";
export type TCloudEnvironment = (typeof clusterContexts)[keyof typeof clusterContexts];
export type TBucketEnvironment = (typeof bucketEnvironments)[keyof typeof bucketEnvironments];
export type TBucketRegion = (typeof bucketRegions)[keyof typeof bucketRegions];
export type TApplicationAssetsBucketConfig = {
    prNumber?: string;
    applicationName: string;
};
export type TApplicationIndexBucketConfig = TApplicationAssetsBucketConfig;
export type TUploadScriptFile = {
    fileName: string;
    fileContent: string;
    filePath: string;
};
type TPaths = {
    publicAssetsPath: string;
    deploymentsPath: string;
    dotenvPath?: string;
    assetsPath: string;
};
export type TStorageProviderTag = 'gs' | 's3';
export type TStorageProvider = {
    tag: TStorageProviderTag;
    urls: {
        public: {
            default: string | null;
        } & Partial<Record<TBucketEnvironment, string>>;
        bucket?: Partial<Record<TBucketRegion, string>>;
    };
};
export type TStorageProviderConfig = {
    bucketRegion?: TBucketRegion;
    bucketEnvironment?: TBucketEnvironment;
};
export interface IStorageProvider {
    getTag(): TStorageProviderTag;
    getBucketRegion(): TBucketRegion | undefined;
    getBucketEnvironment(): TBucketEnvironment | undefined;
    getProtocol(): string;
    getPublicBaseUrl(): string;
    getCdnUrl(config: TApplicationAssetsBucketConfig): string;
    getAssetsBucketUrl(config: TApplicationAssetsBucketConfig): string;
    getApplicationIndexBucketUrl(config: TApplicationIndexBucketConfig): string;
}
export type TUploadScriptsGeneratorConfig = {
    bucketRegion: TBucketRegion;
    bucketEnvironment?: TBucketEnvironment;
};
export interface IUploadScriptsGenerator {
    getProductionBundlesUploadScript(config: TCreateApplicationAssetsUploadScript): string;
    getApplicationIndexUploadScript(config: TCreateApplicationIndexUploadScript): string;
}
export type TCompileEnvApplicationIndexes = {
    cliFlags: TCompileDeploymentsFlags;
    storageProvider: IStorageProvider;
    uploadScriptsGenerator: IUploadScriptsGenerator;
    paths: TPaths;
    cloudEnvironment: TCloudEnvironment;
};
export type TCompileApplicationAssets = {
    cliFlags: TCompileDeploymentsFlags;
    storageProvider: IStorageProvider;
    uploadScriptsGenerator: IUploadScriptsGenerator;
    paths: TPaths;
};
export type TBucketEnvironmentConfig = {
    cloudEnvironment: TCloudEnvironment;
    bucketEnvironment: TBucketEnvironment;
    storageProviders?: TStorageProviderTag[];
};
export type TStorageBucketsConfig = Record<TBucketRegion, TBucketEnvironmentConfig[]>;
export type TCosmiconfigResult = {
    config: TStorageBucketsConfig;
};
export type TCreateApplicationIndexUploadScript = {
    packageManagerName: string;
    storageProvider: IStorageProvider;
    bucketUrl: string;
    cdnUrl: string;
    buildRevision: string;
    buildNumber: number;
    applicationIndexOutFile: string;
};
export type TCreateApplicationAssetsUploadScript = {
    storageProvider: IStorageProvider;
    bucketUrl: string;
    assetsPath: string;
    skipMenu: boolean;
};
export type TLoadDotDev = {
    dotenvPath?: string;
    cloudEnvironment: TCloudEnvironment;
};
export type TLocalizedField = {
    locale: string;
    value: string;
};
export type TCompileDeploymentsFlags = {
    applicationName: string;
    buildRevision: string;
    buildNumber: number;
    dotenvFolder?: string;
    prNumber?: string;
    mcUrl?: string;
    mcApiUrl?: string;
    applicationIndexOutFile: string;
    applicationIndexUploadScriptOutFile: string;
    applicationAssetsUploadScriptOutFile: string;
    ciAssetsRootPath?: string;
    packageManagerName: string;
    skipMenu: boolean;
};
export type TCompileMenuFlags = {
    dotenvFolder?: string;
};
export type TCreateVersionFlags = {
    versionUrl: string;
    buildRevision: string;
    buildNumber: number;
    rollbacks: number;
    outFile?: string;
    packageManagerName: string;
};
export type TValidateMenuFlags = {
    inputFile: string;
    navigation?: string;
};
export type TEvaluateChangeTriggersCliFlags = {
    branch: string;
    baseBranch: string;
    headRevision: string;
    triggerName: string;
    gitCommitMessage?: string;
    outEnvFile?: string;
    silent?: boolean;
};
type TChangeTrigger = {
    name: string;
    containsSharedFiles?: boolean;
    include: string[];
    exclude: string[];
    ignore: string[];
};
export type TChangeTriggersConfig = TChangeTrigger[];
export {};
