import _forEachInstanceProperty from '@babel/runtime-corejs3/core-js-stable/instance/for-each';
import _Object$keys from '@babel/runtime-corejs3/core-js-stable/object/keys';
import _filterInstanceProperty from '@babel/runtime-corejs3/core-js-stable/instance/filter';
import _bindInstanceProperty from '@babel/runtime-corejs3/core-js-stable/instance/bind';
import { cosmiconfig } from 'cosmiconfig';
import { merge } from 'ts-deepmerge';
import lodash from 'lodash';

const clusterContexts = {
  'ctp_staging_gcp_europe-west1_v1': 'ctp_staging_gcp_europe-west1_v1',
  'vw_staging_aws_eu-central-1_v1': 'vw_staging_aws_eu-central-1_v1',
  'ctp_testing_performance_gcp_europe-west1_v1': 'ctp_testing_performance_gcp_europe-west1_v1',
  'ctp_production_aws_eu-central-1_v1': 'ctp_production_aws_eu-central-1_v1',
  'ctp_production_gcp_europe-west1_v1': 'ctp_production_gcp_europe-west1_v1',
  'vw_production_aws_eu-central-1_v1': 'vw_production_aws_eu-central-1_v1',
  'ctp_production_aws_us-east-2_v1': 'ctp_production_aws_us-east-2_v1',
  'ctp_production_gcp_us-central1_v1': 'ctp_production_gcp_us-central1_v1',
  'ctp_production_gcp_australia-southeast1_v1': 'ctp_production_gcp_australia-southeast1_v1'
};
const bucketEnvironments = {
  'ctp-gcp-staging-eu': 'ctp-gcp-staging-eu',
  'ctp-vw-staging-eu': 'ctp-vw-staging-eu',
  'ctp-gcp-testing-performance-eu': 'ctp-gcp-testing-performance-eu',
  'ctp-aws-production-eu': 'ctp-aws-production-eu',
  'ctp-aws-production-us': 'ctp-aws-production-us',
  'ctp-gcp-production-au': 'ctp-gcp-production-au',
  'ctp-gcp-production-eu': 'ctp-gcp-production-eu',
  'ctp-gcp-production-us': 'ctp-gcp-production-us',
  'ctp-vw-production-eu': 'ctp-vw-production-eu',
  'ctp-att-production-va': 'ctp-att-production-va'
};
const bucketRegions = {
  previews: 'merchant-center-previews',
  europe: 'merchant-center-europe',
  northAmerica: 'merchant-center-north-america',
  asia: 'merchant-center-asia'
};
const storageProviders = {
  gs: {
    tag: 'gs',
    urls: {
      public: {
        default: 'https://storage.googleapis.com'
      }
    }
  },
  s3: {
    tag: 's3',
    urls: {
      public: {
        default: null,
        [bucketEnvironments['ctp-aws-production-eu']]: 'https://mc-storage.eu-central-1.aws.commercetools.com',
        [bucketEnvironments['ctp-aws-production-us']]: 'https://mc-storage.us-east-2.aws.commercetools.com'
      },
      bucket: {
        [bucketRegions.europe]: 'commercetools-mc-storage-aws-eu-prod-merchant-center',
        [bucketRegions.northAmerica]: 'commercetools-mc-storage-aws-us-prod-merchant-center'
      }
    }
  }
};

const defaultConfig = {
  [bucketRegions.previews]: [{
    cloudEnvironment: clusterContexts['ctp_staging_gcp_europe-west1_v1'],
    bucketEnvironment: bucketEnvironments['ctp-gcp-staging-eu']
  }, {
    cloudEnvironment: clusterContexts['vw_staging_aws_eu-central-1_v1'],
    bucketEnvironment: bucketEnvironments['ctp-vw-staging-eu']
  }, {
    cloudEnvironment: clusterContexts['ctp_testing_performance_gcp_europe-west1_v1'],
    bucketEnvironment: bucketEnvironments['ctp-gcp-testing-performance-eu']
  }],
  [bucketRegions.europe]: [{
    cloudEnvironment: clusterContexts['ctp_production_aws_eu-central-1_v1'],
    bucketEnvironment: bucketEnvironments['ctp-aws-production-eu'],
    storageProviders: ['gs', 's3']
  }, {
    cloudEnvironment: clusterContexts['ctp_production_gcp_europe-west1_v1'],
    bucketEnvironment: bucketEnvironments['ctp-gcp-production-eu']
  }, {
    cloudEnvironment: clusterContexts['vw_production_aws_eu-central-1_v1'],
    bucketEnvironment: bucketEnvironments['ctp-vw-production-eu']
  }],
  [bucketRegions.northAmerica]: [{
    cloudEnvironment: clusterContexts['ctp_production_aws_us-east-2_v1'],
    bucketEnvironment: bucketEnvironments['ctp-aws-production-us'],
    storageProviders: ['gs', 's3']
  }, {
    cloudEnvironment: clusterContexts['ctp_production_gcp_us-central1_v1'],
    bucketEnvironment: bucketEnvironments['ctp-gcp-production-us']
  }],
  [bucketRegions.asia]: [{
    cloudEnvironment: clusterContexts['ctp_production_gcp_australia-southeast1_v1'],
    bucketEnvironment: bucketEnvironments['ctp-gcp-production-au']
  }]
};
function defineStorageBucketsConfig() {
  let _ref = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
    regions = _ref.regions,
    options = _ref.options;
  const actualConfig = regions ? merge(defaultConfig, regions) : defaultConfig;
  const disabledBucketRegions = options?.disabledBucketRegions ?? [];
  const disabledEnvironments = options?.disabledEnvironments ?? [];
  _forEachInstanceProperty(disabledBucketRegions).call(disabledBucketRegions, disabledBucketRegion => {
    delete actualConfig[disabledBucketRegion];
  });
  const allBucketRegions = _Object$keys(actualConfig);
  _forEachInstanceProperty(disabledEnvironments).call(disabledEnvironments, disabledEnvrionment => {
    _forEachInstanceProperty(allBucketRegions).call(allBucketRegions, bucketRegionToBeRemoved => {
      var _context, _context2;
      actualConfig[bucketRegionToBeRemoved] = ((_context = actualConfig[bucketRegionToBeRemoved]) == null ? void 0 : _bindInstanceProperty(_context2 = Function.call).call(_context2, _filterInstanceProperty(_context), _context))?.(bucketEnvironmentConfig => bucketEnvironmentConfig.cloudEnvironment !== disabledEnvrionment);
    });
  });
  return actualConfig;
}
async function loadStorageBucketsConfig() {
  let explorerResult;
  const storageBucketConfigExplorer = cosmiconfig('storage-buckets', {
    searchStrategy: 'project'
  });
  try {
    // This is the list of the supported cloud environments and their related bucket location.
    explorerResult = await storageBucketConfigExplorer.search();
  } catch (e) {
    throw new Error('Failed to load a storage bucket configuration. Create a cosmiconfig for `storage-buckets` for example `storage-buckets.config.cjs`.');
  }
  if (!explorerResult) {
    throw new Error('Failed loading a storage bucket configuration');
  }
  return explorerResult?.config;
}
async function loadConfig(moduleName, defaultConfig) {
  const configExplorer = cosmiconfig(moduleName, {
    searchStrategy: 'project'
  });
  try {
    const cosmiconfigResult = await configExplorer.search();
    const mergedConfig = lodash.merge(defaultConfig, cosmiconfigResult?.config);
    console.log(`ℹ️ Merged and loaded cosmiconfig file for '${moduleName}'.`);
    return mergedConfig;
  } catch (e) {
    console.warn(e);
    throw new Error(`Failed loading a ${moduleName} configuration. Create a cosmiconfig for '${moduleName}' for example '${moduleName}.config.cjs.'`);
  }
}

export { loadConfig as a, bucketEnvironments as b, clusterContexts as c, defineStorageBucketsConfig as d, bucketRegions as e, loadStorageBucketsConfig as l, storageProviders as s };
